-- ============================================================================
-- == HalonaTools - A collection of functions used to help users monitor     ==
-- ==               their buffs.                                             ==
-- ============================================================================

HTBuffMonitor = LibStub("AceAddon-3.0"):NewAddon("HTBuffMonitor", "AceConsole-3.0", "AceEvent-3.0")
local L       = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- Frame required to permit tooltips to be scanned for temp-buffs like poisons.
local TooltipScanner = CreateFrame('GameTooltip', 'TooltipScanner', WorldFrame, 'GameTooltipTemplate')

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBuffMonitor:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBuffMonitor:OnEnable()

  -- Subscribe to Ready Check events, used to trigger buff checks.
  self:RegisterEvent("READY_CHECK")

  -- Declare the tooltip scanner's owner. Used to check weapon buffs.
  TooltipScanner:SetOwner(WorldFrame, "ANCHOR_NONE")

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBuffMonitor:OnDisable()
end

-- ============================================================================
-- == DoMissingBuffCheck(characterName, channelName)                         ==
-- ==                                                                        ==
-- == This function will check the named character for missing buffs.        ==
-- ============================================================================

function HTBuffMonitor:DoMissingBuffCheck(characterName, channelName)

  local hasSuitableFlask          = HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)
  local hasSuitableBattleElixir   = HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)
  local hasSuitableGuardianElixir = HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)
  local hasSuitableFoodBuff       = HTBuffMonitor:IsBuffedWithSuitableFood(characterName)
  local hasSuitablePoisonBuffs    = HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a flask or two elixirs activated.          --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFlask then

    -- == ---------------------------------------------------------------------
    -- == -- Has guardian elixir but suitable battle elixir missing.         --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and     hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-battle-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- Has battle elixir but suitable guardian elixir missing.         --
    -- == ---------------------------------------------------------------------
    if      hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-guardian-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- No flask, no battle elixir, no guardian elixir.                 --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-no-flask-or-elixirs"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a food-buff activated.                     --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFoodBuff then
    if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
      HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-food"], "CHARACTER_NAME", characterName), channelName, nil, nil)
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has suitable rogue poisons activated.          --
  -- == -----------------------------------------------------------------------
  if UnitClass(characterName) == L["all-data-classname-rogue"] then
    if not hasSuitablePoisonBuffs then
      if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-weapon-poisons"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character buffs are nearing expiry.                      --
  -- == -----------------------------------------------------------------------
  if HTConfig:GetIsEnabledBuffMonitorExpiryTimeCheck(nil) then
     HTBuffMonitor:CheckExpiryTimes(characterName, channelName)
  end

end

-- ============================================================================
-- == IsCharacterBuffedWithSuitableFlask(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable flask activated.                                              ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFlasksArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableBattleElixir(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable battle elixir activated.                                      ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableBattleElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableGuardianElixir(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable guardian elixir activated.                                    ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableGuardianElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableFood(characterName)                                ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable food buff activated.                                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFood(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFoodArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableRoguePoisons(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has       ==
-- == suitable rogue poisons on their weapons.                               ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableRoguePoisons(characterName)

  -- == ----------------------------------------------------------------------
  -- == -- If it is impossible to inspect the player, return the default    --
  -- == -- response. This is currently 'true' because we do not want an     --
  -- == -- inability to inspect to trigger 'false-positives'.               --
  -- == ----------------------------------------------------------------------
  if not CanInspect(characterName) then
    return true
  end

  -- == ----------------------------------------------------------------------
  -- == -- Get the array of 'suitable' rogue poisons.                       --
  -- == ----------------------------------------------------------------------
  local suitableArray = HTBuffMonitor:GetSuitableRoguePoisonsArray()

  -- == ----------------------------------------------------------------------
  -- == -- Initialise a counter for rogue poison buffs.                     --
  -- == ----------------------------------------------------------------------
  local poisonBuffCount = 0

  -- == ----------------------------------------------------------------------
  -- == -- Advise server of imminent character inspection.                  --
  -- == ----------------------------------------------------------------------
  NotifyInspect(characterName)

  -- == ----------------------------------------------------------------------
  -- == -- Check the character's mainhand weapon.                           --
  -- == ----------------------------------------------------------------------
  TooltipScanner:ClearLines()
  TooltipScanner:SetInventoryItem(characterName, 16)

  if TooltipScanner:NumLines() < 1 then
    HTConsole:Debug("tooltipscanner numlines is <1")
  else
    for i,v in ipairs(suitableArray) do
      if HTBuffMonitor:FindInTooltips(v) then
        poisonBuffCount = poisonBuffCount + 1
      end
    end
  end

  -- == ----------------------------------------------------------------------
  -- == -- Check the character's offhand weapon.                           --
  -- == ----------------------------------------------------------------------
  TooltipScanner:ClearLines()
  TooltipScanner:SetInventoryItem(characterName, 17)

  if TooltipScanner:NumLines() < 1 then
    HTConsole:Debug("tooltipscanner numlines is <1")
  else
    for i,v in ipairs(suitableArray) do
      if HTBuffMonitor:FindInTooltips(v) then
        poisonBuffCount = poisonBuffCount + 1
      end
    end
  end

  if poisonBuffCount == 2 then
    return true
  else
    return false
  end

end

-- ============================================================================
-- == FindInTooltips(textToFind)                                             ==
-- ==                                                                        ==
-- == This function simply examines an item's tooltip looking for the text   ==
-- == string supplied.                                                       ==
-- ============================================================================

function HTBuffMonitor:FindInTooltips(textToFind)

  textToFind = HTData:SafeString(textToFind)

  local numTooltipLines = TooltipScanner:NumLines()
  for tooltipLineNumber = 1, numTooltipLines do
    local tooltipTextLine = getglobal('TooltipScannerTextLeft' .. tooltipLineNumber):GetText()
    HTConsole:Debug("FindInTooltips(" .. HTData:SafeString(textToFind) .. ") discovered: [" .. HTData:SafeString(tooltipTextLine) .. "]")
    if tooltipTextLine then
      if string.find(tooltipTextLine, textToFind) then
        return true
      end
    else
      return false
    end
  end

  return false

end

-- ============================================================================
-- == IsBuffedWith(characterName, suitableBuffs)                             ==
-- ==                                                                        ==
-- == This function performs a check on the character identified by the      ==
-- == supplied parameter to determine if that character is currently buffed  ==
-- == with any of the buffs detailed in the supplied 'suitableBuffs' table.  ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWith(characterName, suitableBuffs)

  -- == ----------------------------------------------------------------------
  -- == -- Perform the test.                                                --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName = UnitBuff(characterName, b)
    if buffName then
      for i,v in ipairs(suitableBuffs) do
        if suitableBuffs[i] == buffName then
          return true
        end
      end
    end
  end

  return false

end

-- ============================================================================
-- == CheckExpiryTimes(characterName, channelName)                           ==
-- ==                                                                        ==
-- == This function simply checks the expiry times of the buff-types that    ==
-- == HalonaTools monitors and reports any buffs that will expire within a   ==
-- == prescribed time-limit to the nominated channel.                        ==
-- ============================================================================

function HTBuffMonitor:CheckExpiryTimes(characterName, channelName)

  HTConsole:Debug("HTBuffMonitor:CheckExpiryTimes("..characterName..", "..channelName..")")

  -- == ----------------------------------------------------------------------
  -- == -- Setup the 'suitable' buff array and other required properties.   --
  -- == ----------------------------------------------------------------------
  local suitableFlasksArray          = HTBuffMonitor:GetSuitableFlasksArray()
  local suitableBattleElixirsArray   = HTBuffMonitor:GetSuitableBattleElixirsArray()
  local suitableGuardianElixirsArray = HTBuffMonitor:GetSuitableGuardianElixirsArray()
  local suitableFoodArray            = HTBuffMonitor:GetSuitableFoodArray()
  local suitableRoguePoisonsArray    = HTBuffMonitor:GetSuitableRoguePoisonsArray()

  local timeLimitMinutes             = HTConfig:GetBuffMonitorExpiryTimeLimit(nil)

  local suitableBuffsArray           = {}

  if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFlasksArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableBattleElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableGuardianElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFoodArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableRoguePoisonsArray)
  end

  -- == ----------------------------------------------------------------------
  -- == -- Perform the tests.                                               --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName, buffRank, buffIcon, buffCount, buffDispelType, buffDuration, buffExpires, buffCaster, buffIsStealable = UnitBuff(characterName, b)
    if buffName then
      if buffDuration then
        for i,v in ipairs(suitableBuffsArray) do
          if suitableBuffsArray[i] == buffName then

            remainingTime = HTData:SafeNumber(tonumber(format("%.2f",-1*(GetTime()-buffExpires)/60)))

            HTConsole:Debug("Buff: "..buffName)
            HTConsole:Debug("Remaining Time: "..remainingTime..", timeLimitMinutes: "..HTData:SafeNumber(timeLimitMinutes))

            if remainingTime < timeLimitMinutes then
              local messageText = string.gsub(L["bfm-short-buff-duration-announce"], "CHARACTER_NAME", characterName)
              messageText       = string.gsub(messageText, "BUFF_NAME", buffName)
              messageText       = string.gsub(messageText, "TIME_LIMIT", timeLimitMinutes)

              HTConsole:SendChatMessage(messageText, channelName, nil, nil)

            end
          end
        end
      end
    end
  end

end

-- ============================================================================
-- == GetSuitableFlasksArray()                                               ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == flasks with respect to PVE raid usage.                                 ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFlasksArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79469))  -- Flask of Steelskin.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79470))  -- Flask of the Draconic Mind.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79471))  -- Flask of the Winds.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79472))  -- Flask of Titanic Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(94160))  -- Flask of Flowing Water.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53760))  -- Flask of Endless Rage.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53755))  -- Flask of the Frost Wyrm.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(54212))  -- Flask of Pure Mojo.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53758))  -- Flask of Stoneblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(62380))  -- Lesser Flask of Resistance.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableBattleElixirsArray()                                        ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == battle elixirs with respect to PVE raid usage.                         ==
-- ============================================================================

function HTBuffMonitor:GetSuitableBattleElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79477)) -- Elixir of the Cobra.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79481)) -- Elixir of Impossible Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79632)) -- Elixir of Mighty Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79635)) -- Elixir of the Master.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79468)) -- Ghost Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79474)) -- Elixir of the Naga.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60345)) -- Armor Piercing.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60341)) -- Deadly Strikes.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60344)) -- Expertise.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53749)) -- Guru's Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60346)) -- Lightning Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(28497)) -- Mighty Agility.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53748)) -- Mighty Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(33721)) -- Spellpower Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53746)) -- Wrath Elixir.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableGuardianElixirsArray()                                      ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == guardian elixirs with respect to PVE raid usage.                       ==
-- ============================================================================

function HTBuffMonitor:GetSuitableGuardianElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79480)) -- Elixir of Deep Earth.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79631)) -- Prismatic Elixir.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53763)) -- Wrath - Elixir of Protection.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53747)) -- Wrath - Elixir of Spirit.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60343)) -- Wrath - Mighty Defense.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53751)) -- Wrath - Mighty Fortitude.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53764)) -- Wrath - Mighty Mageblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60347)) -- Wrath - Mighty Thoughts.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableFoodArray()                                                 ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == food related buffs with respect to PVE raid usage.                     ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFoodArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFood() then

  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFood() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(35272)) -- Well Fed
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableRoguePoisonsArray()                                         ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == rogue poisons for end-game content.                                    ==
-- ============================================================================

function HTBuffMonitor:GetSuitableRoguePoisonsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataPoisons() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(2892))  -- Classic -  Deadly Poison
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(6947))  -- Classic -  Instant Poison
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathPoisons() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(3408))  -- Classic -  Crippling Poison
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(5760))  -- Classic -  Mind Numbing Poison

    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57981)) -- Wrath   -  Anaesthetic Poison II
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57970)) -- Wrath   -  Deadly Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57965)) -- Wrath   -  Instant Poison IX
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(57975)) -- Wrath   -  Wound Poison VII
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSpellNameFromSpellInfo(spellId)                                     ==
-- ==                                                                        ==
-- == This function simply returns the name of a spell when supplied with    ==
-- == a spellId by using the standard API function GetSpellInfo(spellId) and ==
-- == retrieving the name from the data returned.                            ==
-- ============================================================================

function HTBuffMonitor:GetSpellNameFromSpellInfo(spellId)

   spellName, spellRank, spellIcon, spellPowerCost, spellIsFunnel, spellPowerType, spellCastingTime, spellMinRange, spellMaxRange = GetSpellInfo(spellId)

   spellName = HTData:SafeString(spellName)

   return spellName

end

-- ============================================================================
-- == READY_CHECK(sender)                                                    ==
-- ==                                                                        ==
-- == This function is triggered when a ready check is fired.                ==
-- ============================================================================

function HTBuffMonitor:READY_CHECK(sender)

  if HTConfig:GetIsEnabledBuffMonitor(nil) then
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-01"], "CHAT", nil, nil)
    HTBuffMonitor:DoSelfCheck()
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-02"], "CHAT", nil, nil)
  end

end

-- ============================================================================
-- == DoHeartbeat()                                                          ==
-- ==                                                                        ==
-- == This function is intended to provide a place where non-critical tasks  ==
-- == that need to be performed periodically can be called from. Typically   ==
-- == each HT component may have an instance of this method and each will    ==
-- == be called in turn by the corresponding function in HTConsole which is  ==
-- == considered to be the master-heartbeat component.                       ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeat()

  HTBuffMonitor:DoHeartbeatCheck()

end

-- ============================================================================
-- == DoHeartbeatCheck()                                                     ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame. It is called via the periodic     ==
-- == heartbeat that occurs (normally) every 5-15 minutes depending upon     ==
-- == configuration.                                                         ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeatCheck()

  if HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil) then
    HTBuffMonitor:DoSelfCheck()
  end

end

-- ============================================================================
-- == DoManualCheck()                                                        ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame.                                   ==
-- ============================================================================

function HTBuffMonitor:DoManualCheck()

  HTConsole:SendChatMessage(L["bfm-readycheck-msg-01"], "CHAT", nil, nil)
  HTBuffMonitor:DoSelfCheck()
  HTConsole:SendChatMessage(L["bfm-readycheck-msg-02"], "CHAT", nil, nil)

end

-- ============================================================================
-- == DoSelfCheck()                                                          ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window.                                                       ==
-- ============================================================================

function HTBuffMonitor:DoSelfCheck()

   local characterName = UnitName("player")

   HTBuffMonitor:DoMissingBuffCheck(characterName, "CHAT")
   HTBuffMonitor:DoMissingBuffCheck(characterName, "ERRORFRAME")

end
