-- ============================================================================
-- == HTData - A collection of helpful data manipulation functions.          ==
-- ============================================================================

HTData = LibStub("AceAddon-3.0"):NewAddon("HTData", "AceConsole-3.0", "AceEvent-3.0")
local L   = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTData:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTData:OnEnable()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTData:OnDisable()
end

-- ============================================================================
-- ==  PadLeadingZeros(number, length)                                       ==
-- ==                                                                        ==
-- == This function pads a number with leading zeros to ensure it represents ==
-- == a string with length matching the supplied length parameter.           ==
-- ============================================================================

function HTData:PadLeadingZeros(number, length)

  local stringLength = string.len(number)
  local padCount     = length - stringLength

  -- HTConsole:Debug("number=[" .. number .. "], length=[" .. length .. "]")
  -- HTConsole:Debug("stringLength=[" .. stringLength .. "], padCount=[" .. padCount .. "]")



  if padCount > 0 then
    for index = 1, padCount, 1 do
      number = "0" .. number

      -- HTConsole:Debug("index=[" .. index .. "], number=[" .. number .. "]")
    end
  end

  return number

end

-- ============================================================================
-- == HTData:StringSplit(text, separator)                                    ==
-- ==                                                                        ==
-- == This function receives a string and converts it into a list of         ==
-- == arguments, using a supplied separator character.                       ==
-- ============================================================================

function HTData:StringSplit(text, separator)

  if text == nil then
    text = ""
  end
   
  if tostring(text) == "true" or tostring(text) == "false" then
    text = ""
  end

  local args = {}
  local i = 0

  -- search for separators in the string and return the separated data
  for value in string.gmatch(text, "[^"..separator.."]+") do
    i = i + 1
    args[i] = value
  end

  return args

end

-- ============================================================================
-- ==  AsResourceName(text)                                                  ==
-- ==                                                                        ==
-- == This function converts the supplied text into a format suitable for    ==
-- == use as all or part of a resource name - that is, the key used to       ==
-- == reference values in the localisation files.                            ==
-- ============================================================================

function HTData:AsResourceName(text)

  text = string.lower(text)
  text = string.gsub(text, " ", "")

  return text

end

-- ============================================================================
-- == SafeString(value)                                                      ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeString(value)

  if value == nil then
    value = ""
  end

  return value

end

-- ============================================================================
-- == SafeNumber(value)                                                      ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeNumber(value)

  if value == nil then
    value = 0
  end

  return value

end

-- ============================================================================
-- == SafeNumericBoolean(value)                                              ==
-- ==                                                                        ==
-- == Checks supplied parameter, replaces unsafe values as required.         ==
-- ============================================================================

function HTData:SafeNumericBoolean(value)

  if value == nil then
    value = "0"
  else
    value = "1"
  end

  return value

end

-- ============================================================================
-- == BooleanAsText(value)                                                   ==
-- ==                                                                        ==
-- == Expresses a boolean as text. True maps to "Yes", false to "No"         ==
-- ============================================================================

function HTData:BooleanAsText(value)

  if value == nil then
    value = true
  end

  if value then
    return L["data-conversion-boolean-as-text-yes"]
  else
    return L["data-conversion-boolean-as-text-no"]
  end

end

-- ============================================================================
-- == JoinTables(table1, table2)                                             ==
-- ==                                                                        ==
-- == This function simply joins two tables together and returns the result. ==
-- ============================================================================

function HTData:JoinTables(table1, table2)

   for k,v in ipairs(table2) do
      table.insert(table1, v)
   end

   return table1

end

-- ============================================================================
-- == IsInTable(dataTable, dataValue)                                        ==
-- ==                                                                        ==
-- == This function indicates whether the supplied value is contained in the ==
-- == supplied table, returning true if it is, false otherwise.              ==
-- ============================================================================

function HTData:IsInTable(dataTable, dataValue)

  for i,v in ipairs(dataTable) do
    if HTData:SafeString(v) == HTData:SafeString(dataValue) then
      return true
    end
  end

  return false

end

-- ============================================================================
-- == BuildStringFromTable(dataTable, dataString, separator)                 ==
-- ==                                                                        ==
-- == This function builds a string from a table by concatenating all values ==
-- == together, using a nominal separator between values. The string being   ==
-- == built is also a parameter allowing the string to contain data before   ==
-- == the concatenation process begins.                                      ==
-- ==                                                                        ==
-- == Example: A table containing names, a dataString of "My friends:" and a ==
-- ==          separator of ", " would give something like...                ==
-- ==                                                                        ==
-- ==          "My friends: Bobby, Charlie, Sue"                             ==
-- ==                                                                        ==
-- ============================================================================

function HTData:BuildStringFromTable(dataTable, dataString, separator)

  for i,v in ipairs(dataTable) do

    if i > 1 then
      dataString = dataString .. separator
    end

    dataString = dataString .. v

  end

  return dataString

end

-- ============================================================================
-- == SecondsToMinutes(seconds)                                              ==
-- ==                                                                        ==
-- == This function converts a value in seconds to a value in minutes and    ==
-- == seconds.                                                               ==
-- ============================================================================

function HTData:SecondsToMinutes(seconds)

  local minutes = math.floor(tonumber(seconds/60))

  seconds = seconds - (minutes * 60)

  local timeString = minutes .. "m" .. seconds .. "s"

  return timeString

end

-- ============================================================================
-- == HTData:GetTimeString()                                                 ==
-- ==                                                                        ==
-- == This function returns a simple HH:MM timestring.                       ==
-- ============================================================================

function HTData:GetTimeString()

  local hours,minutes = GetGameTime();
  local timeString    = HTData:PadLeadingZeros(HTData:SafeNumber(hours), 2) .. ":" .. HTData:PadLeadingZeros(HTData:SafeNumber(minutes), 2)

  return timeString

end

-- ============================================================================
-- == GetFirstWord(text)                                                     ==
-- ==                                                                        ==
-- == This function returns the first word in the supplied text.             ==
-- ============================================================================

function HTData:GetFirstWord(text)

  local firstWord = ""
  local words     = HTData:StringSplit(text, " ")

  if #words > 0 then
    local firstWord = HTData:SafeString(words[1])

  end

  return firstWord

end
